@@ -1,4 +1,5 @@
-# Copyright (c) 2006,2007 Mitch Garnaat http://garnaat.org/
+# Copyright (c) 2006-2009 Mitch Garnaat http://garnaat.org/
+# Copyright (c) 2008 rPath, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -41,7 +42,6 @@ import hmac
 import httplib
 import socket, errno
 import re
-import sha
 import sys
 import time
 import urllib, urlparse
@@ -53,6 +53,35 @@ from boto.resultset import ResultSet
 import boto.utils
 from boto import config, UserAgent, handler
 
+#
+# the following is necessary because of the incompatibilities
+# between Python 2.4, 2.5, and 2.6 as well as the fact that some
+# people running 2.4 have installed hashlib as a separate module
+# this fix was provided by boto user mccormix.
+# see: http://code.google.com/p/boto/issues/detail?id=172
+# for more details.
+#
+try:
+    from hashlib import sha1 as sha
+    from hashlib import sha256 as sha256
+    
+    if sys.version[:3] == "2.4":
+        # we are using an hmac that expects a .new() method.
+        class Faker:
+            def __init__(self, which):
+                self.which = which
+                self.digest_size = self.which().digest_size
+            
+            def new(self, *args, **kwargs):
+                return self.which(*args, **kwargs)
+        
+        sha = Faker(sha)
+        sha256 = Faker(sha256)
+
+except ImportError:
+    import sha
+    sha256 = None
+
 PORTS_BY_SECURITY = { True: 443, False: 80 }
 
 class AWSAuthConnection:
@@ -74,7 +103,10 @@ class AWSAuthConnection:
         @param is_secure: Whether the connection is over SSL
         
         @type https_connection_factory: list or tuple
-        @param https_connection_factory: A pair of an HTTP connection factory and the exceptions to catch. The factory should have a similar interface to L{httplib.HTTPSConnection}.
+        @param https_connection_factory: A pair of an HTTP connection
+                                         factory and the exceptions to catch.
+                                         The factory should have a similar
+                                         interface to L{httplib.HTTPSConnection}.
         
         @type proxy:
         @param proxy:
@@ -117,7 +149,19 @@ class AWSAuthConnection:
             self.port = port
         else:
             self.port = PORTS_BY_SECURITY[is_secure]
-        self.server_name = '%s:%d' % (server, self.port)
+        if self.port == 80:
+            self.server_name = server
+        else:
+            # This unfortunate little hack can be attributed to
+            # a difference in the 2.6 version of httplib.  In old
+            # versions, it would append ":443" to the hostname sent
+            # in the Host header and so we needed to make sure we
+            # did the same when calculating the signature.  In 2.6
+            # it no longer does that.  Hence, this kludge.
+            if sys.version[:3] == "2.6" and self.port in [80, 443]:
+                self.server_name = server
+            else:
+                self.server_name = '%s:%d' % (server, self.port)
         
         if aws_access_key_id:
             self.aws_access_key_id = aws_access_key_id
@@ -134,7 +178,11 @@ class AWSAuthConnection:
             self.aws_secret_access_key = config.get('Credentials', 'aws_secret_access_key')
 
         # initialize an HMAC for signatures, make copies with each request
-        self.hmac = hmac.new(key=self.aws_secret_access_key, digestmod=sha)
+        self.hmac = hmac.new(self.aws_secret_access_key, digestmod=sha)
+        if sha256:
+            self.hmac_256 = hmac.new(self.aws_secret_access_key, digestmod=sha256)
+        else:
+            self.hmac_256 = None
 
         # cache up to 20 connections
         self._cache = boto.utils.LRUCache(20)
@@ -174,12 +222,10 @@ class AWSAuthConnection:
                 "a port, using default"
             self.proxy_port = self.port
         self.use_proxy = (self.proxy != None)
-        if self.use_proxy and self.is_secure:
-            raise AWSConnectionError("Unable to provide secure connection through proxy")
 
     def get_http_connection(self, host, is_secure):
         if host is None:
-            host = '%s:%d' % (self.server, self.port)
+            host = self.server_name
         cached_name = is_secure and 'https://' or 'http://'
         cached_name += host
         if cached_name in self._cache:
@@ -190,10 +236,12 @@ class AWSAuthConnection:
         if self.use_proxy:
             host = '%s:%d' % (self.proxy, int(self.proxy_port))
         if host is None:
-            host = '%s:%d' % (self.server, self.port)
+            host = self.server_name
         boto.log.debug('establishing HTTP connection')
         if is_secure:
-            if self.https_connection_factory:
+            if self.use_proxy:
+                connection = self.proxy_ssl()
+            elif self.https_connection_factory:
                 connection = self.https_connection_factory(host)
             else:
                 connection = httplib.HTTPSConnection(host)
@@ -212,6 +260,39 @@ class AWSAuthConnection:
             self.connection = connection
         return connection
 
+    def proxy_ssl(self):
+        host = '%s:%d' % (self.server, self.port)
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        try:
+            sock.connect((self.proxy, int(self.proxy_port)))
+        except:
+            raise
+        sock.sendall("CONNECT %s HTTP/1.0\r\n" % host)
+        sock.sendall("User-Agent: %s\r\n" % UserAgent)
+        if self.proxy_user and self.proxy_pass:
+            for k, v in self.get_proxy_auth_header().items():
+                sock.sendall("%s: %s\r\n" % (k, v))
+        sock.sendall("\r\n")
+        resp = httplib.HTTPResponse(sock, strict=True)
+        resp.begin()
+
+        if resp.status != 200:
+            # Fake a socket error, use a code that make it obvious it hasn't
+            # been generated by the socket library
+            raise socket.error(-71,
+                               "Error talking to HTTP proxy %s:%s: %s (%s)" %
+                               (self.proxy, self.proxy_port, resp.status, resp.reason))
+
+        # We can safely close the response, it duped the original socket
+        resp.close()
+
+        # Wrap the socket in an SSL socket
+        sslSock = socket.ssl(sock, None, None)
+        h = httplib.HTTPConnection(host)
+        # This is a bit unclean
+        h.sock = httplib.FakeSocket(sock, sslSock)
+        return h
+
     def prefix_proxy_to_path(self, path, host=None):
         path = self.protocol + '://' + (host or self.server) + path
         return path
@@ -248,6 +329,11 @@ class AWSAuthConnection:
                     connection.request(method, path, data, headers)
                     response = connection.getresponse()
                 location = response.getheader('location')
+                # -- gross hack --
+                # httplib gets confused with chunked responses to HEAD requests
+                # so I have to fake it out
+                if method == 'HEAD' and getattr(response, 'chunked', False):
+                    response.chunked = 0
                 if response.status == 500 or response.status == 503:
                     boto.log.debug('received %d response, retrying in %d seconds' % (response.status, 2**i))
                     body = response.read()
@@ -295,10 +381,12 @@ class AWSAuthConnection:
         else:
             headers = headers.copy()
         if not headers.has_key('Content-Length'):
-            headers['Content-Length'] = len(data)
+            headers['Content-Length'] = str(len(data))
         if self.use_proxy:
             path = self.prefix_proxy_to_path(path, host)
-            if self.proxy_user and self.proxy_pass:
+            if self.proxy_user and self.proxy_pass and not self.is_secure:
+                # If is_secure, we don't have to set the proxy authentication
+                # header here, we did that in the CONNECT to the proxy.
                 headers.update(self.get_proxy_auth_header())
         self.add_aws_auth_header(headers, method, auth_path or path)
         return self._mexe(method, path, data, headers, host, sender)
@@ -329,6 +417,14 @@ class AWSQueryConnection(AWSAuthConnection):
                                    is_secure, port, proxy, proxy_port, proxy_user, proxy_pass,
                                    debug,  https_connection_factory)
 
+    def get_utf8_value(self, value):
+        if not isinstance(value, str) and not isinstance(value, unicode):
+            value = str(value)
+        if isinstance(value, unicode):
+            return value.encode('utf-8')
+        else:
+            return value
+
     def calc_signature_0(self, params):
         boto.log.debug('using calc_signature_0')
         hmac = self.hmac.copy()
@@ -336,9 +432,11 @@ class AWSQueryConnection(AWSAuthConnection):
         hmac.update(s)
         keys = params.keys()
         keys.sort(cmp = lambda x, y: cmp(x.lower(), y.lower()))
-        qs = ''
+        pairs = []
         for key in keys:
-            qs += key + '=' + urllib.quote(unicode(params[key]).encode('utf-8')) + '&'
+            val = self.get_utf8_value(params[key])
+            pairs.append(key + '=' + urllib.quote(val))
+        qs = '&'.join(pairs)
         return (qs, base64.b64encode(hmac.digest()))
 
     def calc_signature_1(self, params):
@@ -346,21 +444,48 @@ class AWSQueryConnection(AWSAuthConnection):
         hmac = self.hmac.copy()
         keys = params.keys()
         keys.sort(cmp = lambda x, y: cmp(x.lower(), y.lower()))
-        qs = ''
+        pairs = []
         for key in keys:
             hmac.update(key)
-            val = params[key]
-            if not isinstance(val, str) and not isinstance(val, unicode):
-                val = str(val)
+            val = self.get_utf8_value(params[key])
             hmac.update(val)
-            qs += key + '=' + urllib.quote(unicode(params[key]).encode('utf-8')) + '&'
+            pairs.append(key + '=' + urllib.quote(val))
+        qs = '&'.join(pairs)
         return (qs, base64.b64encode(hmac.digest()))
 
-    def get_signature(self, params):
-        if self.SignatureVersion == '1':
-            t = self.calc_signature_1(params)
-        elif self.SignatureVersion == '0':
+    def calc_signature_2(self, params, verb, path):
+        boto.log.debug('using calc_signature_2')
+        string_to_sign = '%s\n%s\n%s\n' % (verb, self.server_name.lower(),
+                                           path or '/')
+        if self.hmac_256:
+            hmac = self.hmac_256.copy()
+            params['SignatureMethod'] = 'HmacSHA256'
+        else:
+            hmac = self.hmac.copy()
+            params['SignatureMethod'] = 'HmacSHA1'
+        keys = params.keys()
+        keys.sort()
+        pairs = []
+        for key in keys:
+            val = self.get_utf8_value(params[key])
+            pairs.append(urllib.quote(key, safe='') + '=' + urllib.quote(val, safe='-_~'))
+        qs = '&'.join(pairs)
+        boto.log.debug('query string: %s' % qs)
+        string_to_sign += qs
+        boto.log.debug('string_to_sign: %s' % string_to_sign)
+        hmac.update(string_to_sign)
+        b64 = base64.b64encode(hmac.digest())
+        boto.log.debug('len(b64)=%d' % len(b64))
+        boto.log.debug('base64 encoded digest: %s' % b64)
+        return (qs, b64)
+
+    def get_signature(self, params, verb, path):
+        if self.SignatureVersion == '0':
             t = self.calc_signature_0(params)
+        elif self.SignatureVersion == '1':
+            t = self.calc_signature_1(params)
+        elif self.SignatureVersion == '2':
+            t = self.calc_signature_2(params, verb, path)
         else:
             raise BotoClientError('Unknown Signature Version: %s' % self.SignatureVersion)
         return t
@@ -376,8 +501,8 @@ class AWSQueryConnection(AWSAuthConnection):
         params['AWSAccessKeyId'] = self.aws_access_key_id
         params['SignatureVersion'] = self.SignatureVersion
         params['Timestamp'] = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime())
-        qs, signature = self.get_signature(params)
-        qs = path + '?' + qs + 'Signature=' + urllib.quote(signature)
+        qs, signature = self.get_signature(params, verb, path)
+        qs = path + '?' + qs + '&Signature=' + urllib.quote(signature)
         if self.use_proxy:
             qs = self.prefix_proxy_to_path(qs)
         return self._mexe(verb, qs, None, headers)
@@ -395,6 +520,7 @@ class AWSQueryConnection(AWSAuthConnection):
             parent = self
         response = self.make_request(action, params, path)
         body = response.read()
+        boto.log.debug(body)
         if response.status == 200:
             rs = ResultSet(markers)
             h = handler.XmlHandler(rs, parent)
@@ -410,6 +536,7 @@ class AWSQueryConnection(AWSAuthConnection):
             parent = self
         response = self.make_request(action, params, path)
         body = response.read()
+        boto.log.debug(body)
         if response.status == 200:
             obj = cls(parent)
             h = handler.XmlHandler(obj, parent)
@@ -425,6 +552,7 @@ class AWSQueryConnection(AWSAuthConnection):
             parent = self
         response = self.make_request(action, params, path)
         body = response.read()
+        boto.log.debug(body)
         if response.status == 200:
             rs = ResultSet()
             h = handler.XmlHandler(rs, parent)
